import { RequestHandler, Router } from 'express';
import { Services } from '../../app';
import { CreateUser, ReadUser, LoginUser } from '../../functions/user';
import {
  LoginUserSchema,
  CreateUserSchema,
  ReadUserSchema,
  LogoutUserSchema,
} from './schema/user';
import { getId, SchemaValidator } from './middleware';

function LoginHandler(services: Services): RequestHandler {
  const login = LoginUser(services);

  return async (req, res, next) => {
    const user = await login(getId(req), req.body);
    user ? (req.session.user = user) : (req.session.user = null);
    user
      ? res.status(200).send(user)
      : next({ status: 404, message: 'wrong user or password' });
  };
}

function LogoutHandler(services: Services): RequestHandler {
  return async (req, res, next) => {
    if (req.session.user) {
      req.session.user = null;
      res.status(204).send();
    } else {
      next({ message: 'not logged in' });
    }
  };
}

function CreateHandler(services: Services): RequestHandler {
  const createUser = CreateUser(services);

  return async (req, res, next) => {
    const user = await createUser(getId(req), req.body);
    user ? res.status(201).send(user) : next();
  };
}

function ReadHandler(services: Services): RequestHandler {
  const readUser = ReadUser(services);

  return async (req, res, next) => {
    const user = await readUser(getId(req), req.params.uuid);
    user
      ? res.status(200).send(user)
      : next({ status: 404, message: 'user not found' });
  };
}

export function getRoutes(services: Services) {
  const router = Router();

  router.get(
    '/read/:uuid',
    ReadUserSchema(),
    SchemaValidator(),
    ReadHandler(services),
  );

  router.post(
    '/login',
    LoginUserSchema(),
    SchemaValidator(),
    LoginHandler(services),
  );
  router.post(
    '/logout',
    LogoutUserSchema(),
    SchemaValidator(),
    LogoutHandler(services),
  );
  router.post(
    '/create',
    CreateUserSchema(),
    SchemaValidator(),
    CreateHandler(services),
  );

  return router;
}
